bl_info = {
	"name": "Msp GL datafiles",
	"blender": (2, 80, 0),
	"author": "Mikko Rasa",
	"location": "File > Export",
	"description": "Export Msp GL data",
	"category": "Import-Export" }

if "bpy" in locals():
	import imp
	for sub in "animation", "armature", "datafile", "export_animation", "export_armature", "export_camera", "export_material", "export_mesh", "export_object", "export_scene", "export_texture", "material", "mesh", "properties", "util":
		if sub in locals():
			imp.reload(locals()[sub])

import bpy
from bpy_extras.io_utils import ExportHelper

class ExportMspGLBase(ExportHelper):
	show_progress: bpy.props.BoolProperty(name="Show progress", description="Display progress indicator while exporting", default=True)

	def set_extension(self, ext):
		ext_changed = (ext!=self.filename_ext)
		if ext_changed:
			if self.filepath.endswith(self.filename_ext):
				self.filepath = self.filepath[:-len(self.filename_ext)]
			self.filename_ext = ext
		return ext_changed

	def execute(self, context):
		exporter = self.create_exporter()
		self.prepare_exporter(exporter)
		exporter.export_to_file(context, self.filepath)
		return {"FINISHED"}

	def create_exporter(self):
		raise Exception("create_exporter must be overridden")

	def prepare_exporter(self, exporter):
		for k, v in self.as_keywords().items():
			setattr(exporter, k, v)

class ExportMspGLMeshBase(ExportMspGLBase):
	export_all: bpy.props.BoolProperty(name="Export all selected", description="Export all selected objects (use generated filenames)", default=False)
	use_strips: bpy.props.BoolProperty(name="Use strips", description="Combine faces into triangle strips", default=True)
	use_degen_tris: bpy.props.BoolProperty(name="Use degen tris", description="Concatenate triangle strips with degenerate triangles", default=False)

	def draw(self, context):
		self.general_col = self.layout.column()

		col = self.layout.column()
		if len(context.selected_objects)>1:
			col.label(text="Object selection")
			col.prop(self, "export_all")
		col.label(text="Triangle strips")
		col.prop(self, "use_strips")
		col.prop(self, "use_degen_tris")

class ExportMspGLMesh(bpy.types.Operator, ExportMspGLMeshBase):
	bl_idname = "export_mesh.mspgl_mesh"
	bl_label = "Export Msp GL mesh"
	bl_description = "Export one or more meshes in Msp GL format"

	filename_ext = ".mesh"

	def create_exporter(self):
		from .export_mesh import MeshExporter
		return MeshExporter()

class ExportMspGLObject(bpy.types.Operator, ExportMspGLMeshBase):
	bl_idname = "export_mesh.mspgl_object"
	bl_label = "Export Msp GL object"
	bl_description = "Export one or more objects in Msp GL format"

	filename_ext = ".object"

	collection: bpy.props.BoolProperty(name="As a collection", description="Write all data into a single collection file", default=False)
	shared_resources: bpy.props.BoolProperty(name="Shared resources", description="Use global names for resource files to enable sharing", default=True)

	export_lods: bpy.props.BoolProperty(name="Export LoDs", description="Export all levels of detail", default=True)
	use_textures: bpy.props.BoolProperty(name="Use textures", description="Use textures in the exported object", default=True)

	def check(self, context):
		ext_changed = self.set_extension(".mdc" if self.collection else ".object")
		super_result = super().check(context)
		return ext_changed or super_result

	def create_exporter(self):
		from .export_object import ObjectExporter
		return ObjectExporter()

	def draw(self, context):
		super().draw(context)

		self.general_col.prop(self, "export_lods")
		self.general_col.prop(self, "use_textures")

		col = self.layout.column()
		col.label(text="Files")
		col.prop(self, "collection")
		col.prop(self, "shared_resources")

class ExportMspGLArmature(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export.mspgl_armature"
	bl_label = "Export Msp GL armature"
	bl_description = "Export an armature in Msp GL format"

	filename_ext = ".arma"

	def create_exporter(self):
		from .export_armature import ArmatureExporter
		return ArmatureExporter()

class ExportMspGLAnimation(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export.mspgl_animation"
	bl_label = "Export Msp GL animation"
	bl_description = "Export one or more animations in Msp GL format"

	filename_ext = ".anim"

	export_all: bpy.props.BoolProperty(name="Export all animations", description="Export all animations present on the selected objects' NLA tracks")
	collection: bpy.props.BoolProperty(name="As a collection", description="Export the animations as a single collection file", default=True)
	looping_threshold: bpy.props.FloatProperty(name="Looping threshold", description="Tolerance for curve beginning and end values for looping", min=0.0, soft_max=1.0, precision=4, default=0.001)

	def check(self, context):
		ext_changed = self.set_extension(".mdc" if self.export_all and self.collection else ".anim")
		super_result = super().check(context)
		return ext_changed or super_result

	def create_exporter(self):
		from .export_animation import AnimationExporter
		return AnimationExporter()

	def draw(self, context):
		col = self.layout.column()
		col.prop(self, "export_all")
		if self.export_all:
			col.prop(self, "collection")
		col.prop(self, "looping_threshold")

class ExportMspGLScene(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export_scene.mspgl_scene"
	bl_label = "Export Msp GL scene"
	bl_description = "Export the active scene in Msp GL format"

	filename_ext = ".scene"

	selected_only: bpy.props.BoolProperty(name="Selected objects only", description="Only export the selected objects")
	visible_collections: bpy.props.BoolProperty(name="Visible collections only", description="Only export objects in visible collections", default=True)
	resource_collection: bpy.props.BoolProperty(name="Resource collection", description="Put resources to a single collection file", default=True)
	skip_existing: bpy.props.BoolProperty(name="Skip existing files", description="Skip resources that already exist as files", default=True)

	def create_exporter(self):
		from .export_scene import SceneExporter
		return SceneExporter()

	def draw(self, context):
		col = self.layout.column()
		col.prop(self, "selected_only")
		col.prop(self, "visible_collections")
		col.prop(self, "resource_collection")
		if self.resource_collection:
			col.prop(self, "skip_existing")

class ExportMspGLCamera(bpy.types.Operator, ExportMspGLBase):
	bl_idname = "export.mspgl_camera"
	bl_label = "Export Msp GL camera"
	bl_description = "Export a camera in Msp GL format"

	filename_ext = ".camera"

	def create_exporter(self):
		from .export_camera import CameraExporter
		return CameraExporter()

def menu_func_export(self, context):
	self.layout.operator(ExportMspGLMesh.bl_idname, text="Msp GL mesh")
	self.layout.operator(ExportMspGLObject.bl_idname, text="Msp GL object")
	self.layout.operator(ExportMspGLArmature.bl_idname, text="Msp GL armature")
	self.layout.operator(ExportMspGLAnimation.bl_idname, text="Msp GL animation")
	self.layout.operator(ExportMspGLScene.bl_idname, text="Msp GL scene")
	self.layout.operator(ExportMspGLCamera.bl_idname, text="Msp GL camera")

classes = [ExportMspGLMesh, ExportMspGLObject, ExportMspGLArmature, ExportMspGLAnimation, ExportMspGLScene, ExportMspGLCamera]

def register():
	for c in classes:
		bpy.utils.register_class(c)

	bpy.types.TOPBAR_MT_file_export.append(menu_func_export)

	from .properties import register_properties
	register_properties()

def unregister():
	for c in classes:
		bpy.utils.unregister_class(c)

	bpy.types.TOPBAR_MT_file_export.remove(menu_func_export)

	from .properties import unregister_properties
	unregister_properties()
